#ifndef DS18B20_H
#define DS18B20_H

#include "stm32f10x.h"

// 引脚定义
#define DS18B20_DQ_PIN     (1 << 12)  // PD12
#define DS18B20_PORT       GPIOD

// 设置 DQ 引脚为输出模式
#define DQ_OUT()            { \
                            GPIOD->CRH &= ~(0xF << (4 * 4)); /* 清除配置 */ \
                            GPIOD->CRH |= (0x3 << (4 * 4));  /* 设置为推挽输出模式，最大50MHz */ \
                        }

// 设置 DQ 引脚为输入模式
#define DQ_IN()             { \
                            GPIOD->CRH &= ~(0xF << (4 * 4)); /* 清除配置 */ \
                            GPIOD->CRH |= (0x4 << (4 * 4));  /* 设置为浮空输入模式 */ \
                        }

// 获取 DQ 引脚的电平变化
#define DQ_GET()            ((GPIOD->IDR & DS18B20_DQ_PIN) ? 1 : 0)

// DQ 输出高电平或低电平
#define DQ(x)               ((x) ? (GPIOD->BSRR = DS18B20_DQ_PIN) : (GPIOD->BRR = DS18B20_DQ_PIN))


uint8_t DS18B20_Check(void);
char DS18B20_Init(void);
void DS18B20_Start(void);

float DS18B20_GetTemperture(void);

#endif // DS18B20_H
