#include "onewire.h"
#include "common.h"


// 初始化 One-Wire 总线 (PD12)
void Onewire_Init(void) {

    // 设置高电平
    ONEWIRE_PORT->BSRR = ONEWIRE_PIN;
}

// 设置引脚为输出模式
void Onewire_Pin_Output(void) {
    // 设置PD12为推挽输出模式
    GPIOD->CRH &= ~(0xF << 16);   // 清除PD12的配置
    GPIOD->CRH |= (0x3 << 16);    // 设置为推挽输出模式
}

// 设置引脚为输入模式
void Onewire_Pin_Input(void) {
    // 设置PD12为浮空输入模式
    GPIOD->CRH &= ~(0xF << 16);   // 清除PD12的配置
    GPIOD->CRH |= (0x4 << 16);    // 设置为浮空输入模式
}

// 复位 One-Wire 总线并检测传感器是否存在
uint8_t Onewire_Reset(void) {
    uint8_t response = 0;

    // 拉低总线 480 微秒
    Onewire_Pin_Output();
    ONEWIRE_PORT->BRR = ONEWIRE_PIN;  // 拉低
    Delay(480, 'u');

    // 释放总线，并等待响应
    Onewire_Pin_Input();
    Delay(60, 'u');

    // 检测传感器响应
    response = (ONEWIRE_PORT->IDR & ONEWIRE_PIN) ? 1 : 0;

    // 等待 420 微秒
    Delay(420, 'u');

    return !response;  // 返回响应信号，0 表示设备存在
}

// 向 One-Wire 设备发送一位数据
void Onewire_SendBit(unsigned char Bit) {
    Onewire_Pin_Output();  // 设置为输出模式

    if (Bit) {
        ONEWIRE_PORT->BRR = ONEWIRE_PIN;  // 拉低 1 微秒
        Delay(1, 'u');
        Onewire_Pin_Input();  // 释放总线
        Delay(60, 'u');  // 保持高电平 60 微秒
    } else {
        ONEWIRE_PORT->BRR = ONEWIRE_PIN;  // 拉低 60 微秒
        Delay(60, 'u');
        Onewire_Pin_Input();  // 释放总线
    }
}

// 从 One-Wire 设备接收一位数据
unsigned char Onewire_ReceiveBit(void) {
    unsigned char Bit = 0;

    Onewire_Pin_Output();  // 设置为输出模式
    ONEWIRE_PORT->BRR = ONEWIRE_PIN;  // 拉低 1 微秒
    Delay(1, 'u');

    Onewire_Pin_Input();  // 释放总线
    Delay(14, 'u');

    // 读取数据
    if (ONEWIRE_PORT->IDR & ONEWIRE_PIN) {
        Bit = 1;
    }

    Delay(45, 'u');  // 等待剩余时间

    return Bit;
}

// 向 One-Wire 设备发送一个字节数据
void Onewire_SendByte(unsigned char Byte) {
    for (unsigned char i = 0; i < 8; i++) {
        Onewire_SendBit(Byte & 0x01);  // 发送每一位
        Byte >>= 1;
    }
}

// 从 One-Wire 设备接收一个字节数据
unsigned char Onewire_ReceiveByte(void) {
    unsigned char Byte = 0;

    for (unsigned char i = 0; i < 8; i++) {
        Byte >>= 1;
        if (Onewire_ReceiveBit()) {
            Byte |= 0x80;  // 设置最高位
        }
    }

    return Byte;
}
