#include  "temperature_sensor.h"
#include  "onewire.h"
#include "uart_log.h"
#include <stddef.h>  // 引入 size_t 类型定义
#define CRC8_POLYNOMIAL 0x31

uint8_t crc8(const uint8_t *buf, size_t len) {
    uint8_t crc = 0;
    for (size_t pos = 0; pos < len; ++pos) {
        crc ^= buf[pos];
        for (uint8_t bit = 8; bit; --bit) {
            if (crc & 0x80) {
                crc = (crc << 1) ^ CRC8_POLYNOMIAL;
            } else {
                crc = (crc << 1);
            }
        }
    }
    return crc;
}

void TemperatureSensor_ConvertT(void) {
    Onewire_Init();  // 初始化 One-Wire 总线
    if (!Onewire_Reset()) {  // 复位总线并检查是否有设备响应
        Serial_Printf("No device detected.\n");
        return;
    }

    // 跳过 ROM
    Onewire_SendByte(0xCC);

    // 启动温度转换
    Onewire_SendByte(0x44);  // 启动温度转换命令
}

float TemperatureSensor_ReadT(void) {
    // uint8_t rom_address[8];  // 存储ROM地址  如果不需要，可以移除这一行
    uint8_t scratchpad[9];   // 存储Scratchpad数据
    uint8_t crc_result;
    int16_t Temp;
    float temperature;

    Onewire_Init();  // 初始化 One-Wire 总线
    if (!Onewire_Reset()) {  // 复位总线并检查是否有设备响应
        Serial_Printf("No device detected.\n");
        return -1.0;
    }

    // 跳过 ROM
    Onewire_SendByte(0xCC);

    // 读取 Scratchpad
    Onewire_SendByte(0xBE);

    // 读取 Scratchpad 数据
    for (int i = 0; i < 9; i++) {
        scratchpad[i] = Onewire_ReceiveByte();
    }

    // 计算 CRC 并验证
    crc_result = crc8(scratchpad, 8);
    if (crc_result != scratchpad[8]) {
        Serial_Printf("CRC error.\n");
        return -1.0;
    }

    // 提取温度数据
    uint8_t TLSB = scratchpad[0];  // 低字节
    uint8_t TMSB = scratchpad[1];  // 高字节

    Temp = (TMSB << 8) | TLSB;  // 合并两个字节
    temperature = Temp / 16.0;   // 计算实际温度

    Serial_Printf("TLSB: %02X, TMSB: %02X, Temp: %d, Temperature: %.2f C\n",
                  TLSB, TMSB, Temp, temperature);

    return temperature;
}
